/* SpinCore Technologies, Inc
 * http://www.spincore.com
 * Last modified 2004-08-31
 */

/* OS dependent functions for Linux. Link this file with your 
 * program if you are compiling on linux. Note: if you are using 
 * gcc and you get link errors concerning outb, outl, inb, or inl
 * functions, you need to compile using optimizations. (for 
 * example using flags like -O or -O2)
 */

#include <stdio.h>
#include <sys/io.h>
#include "os.h"

static int base_address;

/* pb_init ()
 * Initialize the board. This function should be called before 
 * any other board controlling functions are used. If the board 
 * is successfully initiazed, 0 is returned. On error, -1 is
 * returned.
 */
int pb_init ()
{

  /* The value of base address must be manual configured at this time. To determine what the base address of the board is, run the command "lspci -v" and look for the line containing "Applied Micro Circuits Corp.: Unknown Device 8852". Just below this line there is a line saying "I/O ports at xxxx" where xxxx is the base address of the board. The variable base_address below should be set to this value. The value will not change unless you add, remove, or rearrange the PCI cards in your computer. */

  base_address = 0xec00;
  
  /* Gain permission to write to IO ports */
  if (iopl (3)) {
    printf ("Error: couldnt get io permssions.\n");
    return -1;
  }

  return 0;
}

/* pb_close ()
 * Release control of the board. This should be called when 
 * your control program is done. Returns 0 on success and
 * -1 on failure
 */
int pb_close ()
{
  return 0;
}

/* write_word ()
 * Write a 4 byte word to the given IO port. Returns 0 on
 * success, -1 otherwise.
 */
int write_word (unsigned int data, unsigned int port)
{
  outl (data, base_address + port);

  return 0;
}

/* write_byte ()
 * Write 1 byte to the given IO port. Returns 0 on success,
 * -1 otherwise.
 */
int write_byte (char data, unsigned int port)
{
  outb (data, base_address + port);
  
  return 0;
}

/* read_word ()
 * Read 4 byte word from the given port.
 */
unsigned int read_word (unsigned int port)
{
  return inl (base_address + port);
}

/* read_byte ()
 * Read 1 byte from the given port
 */
char read_byte (unsigned int port)
{
  return inb (base_address + port);
}


